@@ -9,7 +9,7 @@ from mrphy import γH, dt0, π
 from mrphy import utils, beffective
 
 
-__all__ = ['blochsim_1step', 'blochsim', 'blochsim_ab']
+__all__ = ['blochsim_1step', 'blochsim', 'blochsim_ab', 'freeprec']
 
 
 def blochsim_1step(
@@ -121,3 +121,46 @@ def blochsim_ab(M: Tensor, A: Tensor, B: Tensor) -> Tensor:
     """
     M = (A @ M[..., None]).squeeze_(dim=-1) + B
     return M
+
+
+def freeprec(
+    M: Tensor, dur: Tensor, *,
+    T1: Optional[Tensor] = None, T2: Optional[Tensor] = None,
+    Δf: Optional[Tensor] = None
+) -> Tensor:
+    r"""Isochromats free precession with given relaxation and off-resonance
+
+    Usage:
+        ``M = freeprec(M, dur, *, T1, T2, Δf)``
+    Inputs:
+        - ``M``: `(N, *Nd, xyz)`, Magnetic spins, assumed equilibrium \
+          magnitude [0 0 1]
+        - ``dur``: `()` ⊻ `(N ⊻ 1,)`, "Sec", duration of free-precession.
+    OPTIONALS:
+        - ``T1``: `()` ⊻ `(N ⊻ 1, *Nd ⊻ 1,)`, "Sec", T1 relaxation.
+        - ``T2``: `()` ⊻ `(N ⊻ 1, *Nd ⊻ 1,)`, "Sec", T2 relaxation.
+        - ``Δf``: `(N ⊻ 1, *Nd ⊻ 1,)`, "Hz", off-resonance.
+    Outputs:
+        - ``M``: `(N, *Nd, xyz)`, Result magnetic spins
+    """
+    ndim = M.ndim  # dur, T1, T2, Δf are reshaped to be compatible w/ M
+    dur = dur.reshape(dur.shape+(ndim-dur.ndim)*(1,))
+
+    Mx, My, Mz = M.split(1, dim=-1)  # (N, *Nd, 1)
+
+    # Precession
+    if Δf is not None:
+        Δf = Δf.reshape(Δf.shape+(ndim-Δf.ndim)*(1,))
+        ϕ = -(2*π)*Δf*dur  # positive Δf dephases spin clock-wise/negatively
+        cϕ, sϕ = torch.cos(ϕ), torch.sin(ϕ)
+        Mx, My = cϕ*Mx-sϕ*My, sϕ*Mx+cϕ*My
+
+    # Relaxation
+    assert((T1 is None) == (T2 is None))  # both or neither
+    if T1 is not None:
+        T1, T2 = (x.reshape(x.shape+(ndim-x.ndim)*(1,)) for x in (T1, T2))
+        E1, E2 = torch.exp(-dur/T1), torch.exp(-dur/T2)
+        Mx, My, Mz = E2*Mx, E2*My, E1*Mz+1-E1
+
+    M = torch.cat((Mx, My, Mz), dim=-1)  # (N, *Nd, xyz)
+    return M
